/*
Copyright (C) BABEC. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/
package org.chainweaver.did.sdk;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.reflect.TypeToken;
import org.chainweaver.did.sdk.exception.ChainWeaverException;
import org.chainweaver.did.sdk.model.*;
import org.chainweaver.did.sdk.model.req.*;
import org.chainweaver.did.sdk.model.resp.*;
import org.chainweaver.did.sdk.util.HttpUtils;
import org.chainweaver.did.sdk.util.SdkUtils;

import java.io.IOException;
import java.lang.reflect.Type;
import java.util.*;


public class DidClient {

    // DID 管理台服务端版本
    public static final String version12 = "v1.2.x";
    public static final String version11 = "v1.1.x";
    public static final String version10 = "v1.0.x";
    /**
     * 服务访问地址
     */
    private String host = "http://127.0.0.1:1234";
    private String version;

    public String getHost() {
        return host;
    }

    public void setHost(String host) {
        this.host = host;
    }

    private int readTimeOut = 10000;

    /**
     * 连接超时时间
     */
    private int connectTimeOut = 10000;

    private Boolean keepAlive = false;

    public int getReadTimeOut() {
        return readTimeOut;
    }

    public void setReadTimeOut(int readTimeOut) {
        this.readTimeOut = readTimeOut;
    }

    public int getConnectTimeOut() {
        return connectTimeOut;
    }

    public void setConnectTimeOut(int connectTimeOut) {
        this.connectTimeOut = connectTimeOut;
    }

    public DidClient() {
    }

    public DidClient(String host) {
        this.host = host;
        this.version = version11;
    }

    public DidClient(String host, String version) {
        this.host = host;
        this.version = version;
    }

    public DidClient(String host, Boolean keepAlive) {
        this.host = host;
        this.keepAlive = keepAlive;
    }


    public DidClient(String host, int readTimeOut, int connectTimeOut) {
        this.host = host;
        this.readTimeOut = readTimeOut;
        this.connectTimeOut = connectTimeOut;
    }

    public DidClient(String host, int readTimeOut, int connectTimeOut, Boolean keepAlive) {
        this.host = host;
        this.readTimeOut = readTimeOut;
        this.connectTimeOut = connectTimeOut;
        this.keepAlive = keepAlive;
    }

    private String token;

    private Map<String, String> headers = new HashMap<>();
    ;


    private Map<String, String> getHeaders() {
        Map<String, String> headers = new HashMap<>(this.headers);
        headers.put("x-request-id", UUID.randomUUID().toString().replace("-", ""));
        return headers;
    }


    /**
     * 登录
     *
     * @param phoneNumber 用户手机号
     * @param password    用户密码
     * @return {@link AccessToken} data
     * @throws ChainWeaverException 访问异常
     */
    public Response<AccessToken> login(String phoneNumber, String password) throws ChainWeaverException {
        if (version10.equals(this.version)) {
            return loginWithUsername(phoneNumber, password);
        }
        try {
            return loginWithPhone(phoneNumber, password);
        } catch (ChainWeaverException e) {
            return loginWithUsername(phoneNumber, password);
        }
    }

    /**
     * 登录
     *
     * @param username 用户名称（适配0.5版本）
     * @param password 用户密码
     * @return {@link AccessToken} data
     * @throws ChainWeaverException 访问异常
     */
    private Response<AccessToken> loginWithUsername(String username, String password) throws ChainWeaverException {
        Map<String, String> loginMap = new HashMap<>();
        loginMap.put("username", username);
        loginMap.put("password", password);
        Gson gson = new GsonBuilder()
                .registerTypeAdapter(AccessToken.class, new AccessToken.TokenDeserializer())
                .create();
        Response<AccessToken> accessToken;
        try {
            String res = HttpUtils.post(host + DidConfig.login, gson.toJson(loginMap), new HashMap<>(), readTimeOut, connectTimeOut);
            Type type = new TypeToken<Response<AccessToken>>() {
            }.getType();
            accessToken = gson.fromJson(res, type);
            if (accessToken != null && accessToken.getCode() == 200000) {
                this.token = accessToken.getData().getAccessToken();
                this.headers.put("x-token", this.token);
                if (this.keepAlive) {
                    TokenTask.getInstance(username, password, this).startTokenTask(accessToken.getData().getExpiresIn());
                }
                return accessToken;
            }
        } catch (Exception e) {
            throw new ChainWeaverException(e);
        }
        return accessToken;
    }


    private Response<AccessToken> loginWithPhone(String phoneNumber, String password) throws ChainWeaverException {
        Map<String, String> loginMap = new HashMap<>();
        loginMap.put("phoneNumber", phoneNumber);
        loginMap.put("username", phoneNumber);
        loginMap.put("password", Base64.getEncoder().encodeToString(password.getBytes()));
        Gson gson = new GsonBuilder()
                .registerTypeAdapter(AccessToken.class, new AccessToken.TokenDeserializer())
                .create();
        Response<AccessToken> accessToken;
        try {
            String res = HttpUtils.post(host + DidConfig.login, gson.toJson(loginMap), new HashMap<>(), readTimeOut, connectTimeOut);
            Type type = new TypeToken<Response<AccessToken>>() {
            }.getType();
            accessToken = gson.fromJson(res, type);
            if (accessToken != null && accessToken.getCode() == 200000) {
                this.token = accessToken.getData().getAccessToken();
                this.headers.put("x-token", this.token);
                if (this.keepAlive) {
                    TokenTask.getInstance(phoneNumber, password, this).startTokenTask(accessToken.getData().getExpiresIn());
                }
                return accessToken;
            }
        } catch (Exception e) {
            throw new ChainWeaverException(e);
        }
        return accessToken;
    }


    /**
     * 健康检查
     *
     * @return {@link DidHealth} 包含版本号和系统名称
     * @throws ChainWeaverException 访问异常
     */
    public Response<DidHealth> health() throws ChainWeaverException {
        Gson gson = new Gson();
        Response<DidHealth> didHealthResponse;
        try {
            String res = HttpUtils.sendGet(host + DidConfig.health, readTimeOut, connectTimeOut);
            Type type = new TypeToken<Response<DidHealth>>() {
            }.getType();
            didHealthResponse = gson.fromJson(res, type);
            return didHealthResponse;
        } catch (IOException e) {
            e.printStackTrace();
            throw new ChainWeaverException(e);
        }
    }


    /**
     * 使用公钥加密
     *
     * @param plaintext {@link Plaintext#Plaintext(String plaintext, String verificationMethod)}
     *                  <ul><li>plaintext签名原文</li>
     *                  <li>verificationMethod 公钥地址, 默认使用DID第一个公钥</li>
     *                  <li>type 可以不进行设置, 默认为SM2C1C3C2</li></ul>
     * @return {@link Cipher} 加密签名
     * @throws ChainWeaverException 访问异常
     */
    public Response<Cipher> msgEncrypt(Plaintext plaintext) throws ChainWeaverException {
        Gson gson = new Gson();
        Response<Cipher> cipher;
        try {
            String res = HttpUtils.post(host + DidConfig.msgEncrypt, gson.toJson(plaintext), this.getHeaders(), readTimeOut, connectTimeOut);
            Type type = new TypeToken<Response<Cipher>>() {
            }.getType();
            cipher = gson.fromJson(res, type);
            return cipher;
        } catch (IOException e) {
            e.printStackTrace();
            throw new ChainWeaverException(e);
        }
    }

    /**
     * 使用私钥解密
     *
     * @param cipher {@link Cipher#Cipher(String ciphertext)} ciphertext 密文值
     *               <ul><li>verificationMethod 公钥地址，默认使用DID第一个公钥</li>
     *               <li>type 可以不进行设置, 默认为SM2C1C3C2</li></ul>
     * @return {@link Plaintext} 返回原文
     * @throws ChainWeaverException 访问异常
     */
    public Response<Plaintext> msgDecrypt(Cipher cipher) throws ChainWeaverException {
        Gson gson = new Gson();
        Response<Plaintext> plaintextRes;
        try {
            String res = HttpUtils.post(host + DidConfig.msgDecrypt, gson.toJson(cipher), this.getHeaders(), readTimeOut, connectTimeOut);
            Type type = new TypeToken<Response<Plaintext>>() {
            }.getType();
            plaintextRes = gson.fromJson(res, type);
            return plaintextRes;
        } catch (IOException e) {
            throw new ChainWeaverException(e);
        }
    }


    /**
     * 公钥验签
     *
     * @param plaintext {@link Plaintext} 签名对象
     * @return 验签是否正常
     * @throws ChainWeaverException 访问异常
     */
    public Response<Boolean> msgVerify(Plaintext plaintext) throws ChainWeaverException {
        Gson gson = new Gson();
        Response<Boolean> response;
        try {
            String res = HttpUtils.post(host + DidConfig.msgVerify, gson.toJson(plaintext), this.getHeaders(), readTimeOut, connectTimeOut);
            Type type = new TypeToken<Response<Boolean>>() {
            }.getType();
            response = gson.fromJson(res, type);
            return response;
        } catch (IOException e) {
            throw new ChainWeaverException(e);
        }
    }

    /**
     * did私钥签名
     *
     * @param plaintext {@link Plaintext#Plaintext(String plaintext, String verificationMethod)}
     *                  <ul><li>plaintext 签名原文</li>
     *                  <li>verificationMethod 公钥地址</li></ul>
     * @return 返回签名值
     * @throws ChainWeaverException 访问异常
     */
    public Response<Plaintext> msgSign(Plaintext plaintext) throws ChainWeaverException {
        Gson gson = new Gson();
        Response<Plaintext> plaintextRes;
        try {
            String res = HttpUtils.post(host + DidConfig.msgSign, gson.toJson(plaintext), this.getHeaders(), readTimeOut, connectTimeOut);
            Type type = new TypeToken<Response<Plaintext>>() {
            }.getType();
            plaintextRes = gson.fromJson(res, type);
            return plaintextRes;
        } catch (IOException e) {
            throw new ChainWeaverException(e);
        }
    }

    /**
     * 验证vp
     *
     * @param vp {@link VerifiablePresentation }
     * @return 验证是否成功
     * @throws ChainWeaverException 访问异常
     */
    public Response<Boolean> vpVerify(VerifiablePresentation vp) throws ChainWeaverException {
        return this.vpVerify(new Gson().toJson(vp));
    }

    /**
     * 验证vp
     *
     * @param vp vp格式的字符串
     * @return 验证是否成功
     * @throws ChainWeaverException 访问异常
     */
    public Response<Boolean> vpVerify(String vp) throws ChainWeaverException {
        Gson gson = new Gson();
        Response<Boolean> response;
        try {
            String res = HttpUtils.post(host + DidConfig.vpVerify, vp, this.getHeaders(), readTimeOut, connectTimeOut);
            Type type = new TypeToken<Response<Boolean>>() {
            }.getType();
            response = gson.fromJson(res, type);
            return response;
        } catch (IOException e) {
            throw new ChainWeaverException(e);
        }

    }

    /**
     * 签名vp
     *
     * @param vp {@link VerifiablePresentation# VerifiablePresentation(List context, String id, List verifiableCredential, String presentationUsage, String expirationDate, String verifier, List proof)}
     *           <ul><li>context 上下文; 可选 </li>
     *           <li>id 唯一编号; 可选 </li>
     *           <li>verifiableCredential vcId列表,可验证凭据, 与extend至少有一个不为空; 可选 </li>
     *           <li>extend 自定义参数; 可选 </li>
     *           <li>presentationUsage 用途;可选 </li>
     *           <li>expirationDate 过期时间;可选 </li>
     *           <li>verifier 验证人;可选 </li>
     *           <li>proof 证明结构 {@link Proof#Proof(String type, String created, String proofPurpose, String verificationMethod)}</li>
     *                                      <ul><li>type 签名算法; 可选 ,default: SM2Signature</li>
     *                                      <li>created 签名时间; 可选 </li>
     *                                      <li>proofPurpose 签名目的; 可选 .固定：authentication</li>
     *                                      <li>verificationMethod 签名公钥</li></ul></ul>
     * @return vp相关信息
     * @throws ChainWeaverException 访问异常
     */
    public <T> Response<VerifiablePresentation<T>> vpSign(VerifiablePresentation vp, Class<T> tClass) throws ChainWeaverException {
        if (Objects.equals(vp.getId(), "")) {
            vp.setId(SdkUtils.randomId());
        }
        Gson gson = new Gson();
        Response<VerifiablePresentation<T>> vpRes;
        try {
            String res = HttpUtils.post(host + DidConfig.vpSign, gson.toJson(vp), this.getHeaders(), readTimeOut, connectTimeOut);
            Type type = new TypeToken<Response<VerifiablePresentation<T>>>() {
            }.getType();
            vpRes = gson.fromJson(res, type);
            if (vpRes == null || vpRes.getData() == null) {
                return vpRes;
            }
            String extend = gson.toJson(vpRes.getData().getExtend());
            T t = gson.fromJson(extend, tClass);
            vpRes.getData().setExtend(t);
            return vpRes;
        } catch (IOException e) {
            e.printStackTrace();
            throw new ChainWeaverException(e);
        }
    }

    /**
     * 查询DID文档
     *
     * @param did did名称
     * @return Document
     * @throws ChainWeaverException 访问异常
     */
    public Response<Document> documentGet(String did) throws ChainWeaverException {
        Map<String, String> didMap = new HashMap<>();
        didMap.put("did", did);
        Gson gson = new Gson();
        Response<Document> document;
        try {
            String res = HttpUtils.post(host + DidConfig.documentGet, gson.toJson(didMap), this.getHeaders(), readTimeOut, connectTimeOut);
            Type type = new TypeToken<Response<Document>>() {
            }.getType();
            document = gson.fromJson(res, type);
            return document;
        } catch (IOException e) {
            throw new ChainWeaverException(e);
        }
    }


    /**
     * 颁发凭证
     *
     * @param issueVc {@link IssueVc#IssueVc(String templateId, Object credentialSubject, String holder, String verificationMethod) } 对象
     * @return Response
     * @throws ChainWeaverException 访问异常
     */
    public Response<VerifiableCredential> vcIssue(IssueVc issueVc) throws ChainWeaverException {
        Gson gson = new Gson();
        Response<VerifiableCredential> vc;
        try {
            String res = HttpUtils.post(host + DidConfig.vcIssue, gson.toJson(issueVc), this.getHeaders(), readTimeOut, connectTimeOut);
            Type type = new TypeToken<Response<VerifiableCredential>>() {
            }.getType();
            vc = gson.fromJson(res, type);
            return vc;
        } catch (IOException e) {
            throw new ChainWeaverException(e);
        }
    }

    /**
     * 吊销VC
     *
     * @param id 凭证id
     * @return 吊销是否成功
     * @throws ChainWeaverException 访问异常
     */
    public Response<Boolean> vcRevoke(String id) throws ChainWeaverException {
        Map<String, String> revokeMap = new HashMap<>();
        revokeMap.put("id", id);
        Gson gson = new Gson();
        Response<Boolean> response;
        try {
            String res = HttpUtils.post(host + DidConfig.vcRevoke, gson.toJson(revokeMap), this.getHeaders(), readTimeOut, connectTimeOut);
            Type type = new TypeToken<Response<Boolean>>() {
            }.getType();
            response = gson.fromJson(res, type);
            return response;
        } catch (IOException e) {
            throw new ChainWeaverException(e);
        }

    }

    /**
     * 查询已颁发vc
     *
     * @param holder {@link Holder#Holder(String, Integer, Integer)} 持有人对象, 页数, 条数
     * @return vc 列表
     * @throws ChainWeaverException 访问异常
     */
    public Response<RespPage<VerifiableCredential>> vcListIssued(Holder holder) throws ChainWeaverException {
        Gson gson = new Gson();
        Response<RespPage<VerifiableCredential>> vcResp;
        try {
            String res = HttpUtils.post(host + DidConfig.vcIssuedList, gson.toJson(holder), this.getHeaders(), readTimeOut, connectTimeOut);
            Type type = new TypeToken<Response<RespPage<VerifiableCredential>>>() {
            }.getType();
            vcResp = gson.fromJson(res, type);
            return vcResp;
        } catch (IOException e) {
            throw new ChainWeaverException(e);
        }
    }

    /**
     * 查询已吊销vc
     *
     * @param holder {@link Holder#Holder(String, Integer, Integer)} 持有人对象, 页数, 条数
     * @return 吊销vc列表
     * @throws ChainWeaverException 访问异常
     */
    public Response<RespPage<VerifiableCredential>> vcListRevoked(Holder holder) throws ChainWeaverException {
        Gson gson = new Gson();
        Response<RespPage<VerifiableCredential>> vcResp;
        try {
            String res = HttpUtils.post(host + DidConfig.vcRevokedList, gson.toJson(holder), this.getHeaders(), readTimeOut, connectTimeOut);
            Type type = new TypeToken<Response<RespPage<VerifiableCredential>>>() {
            }.getType();
            vcResp = gson.fromJson(res, type);
            return vcResp;
        } catch (IOException e) {
            throw new ChainWeaverException(e);
        }
    }

    /**
     * 用户查询自身vc
     *
     * @param vp {@link VerifiablePresentation#VerifiablePresentation(List verifiableCredential, List proof)}
     *           <ul><li>context 上下文; 可选 </li>
     *           <li>id 唯一编号; 可选 </li>
     *           <li>verifiableCredential vcId列表,可验证凭据, 与extend至少有一个不为空; 可选 </li>
     *           <li>extend 自定义参数; 可选 </li>
     *                  <ul><li>page: 分页结构 {@link Page}</li>
     *                  <li>issuer: 签发者</li>
     *                  <li>templateId：模板ID;可选 </li>
     *                  <li>status: 状态;可选 </li></ul>
     *           <li>presentationUsage 用途;可选 </li>
     *           <li>expirationDate 过期时间;可选 </li>
     *           <li>verifier 验证人;可选 </li>
     *           <li>proof 证明结构 {@link Proof#Proof(String type, String created, String proofPurpose, String verificationMethod)}</li>
     *                                      <ul><li>type 签名算法; 可选 ,default: SM2Signature</li>
     *                                      <li>created 签名时间; 可选 </li>
     *                                      <li>proofPurpose 签名目的; 可选 .固定：authentication</li>
     *                                      <li>verificationMethod 签名公钥</li></ul></ul>
     * @return vc列表
     * @throws ChainWeaverException 访问异常
     */
    public Response<RespPage<VerifiableCredential>> vcList(VerifiablePresentation vp) throws ChainWeaverException {
        Gson gson = new Gson();
        Response<RespPage<VerifiableCredential>> vcResp;
        try {
            String res = HttpUtils.post(host + DidConfig.vcList, gson.toJson(vp), this.getHeaders(), readTimeOut, connectTimeOut);
            Type type = new TypeToken<Response<RespPage<VerifiableCredential>>>() {
            }.getType();
            vcResp = gson.fromJson(res, type);
            return vcResp;
        } catch (IOException e) {
            throw new ChainWeaverException(e);
        }
    }

    /**
     * 根据id查询vc
     *
     * @param vcId vcId
     * @return vc详情
     * @throws ChainWeaverException 访问异常
     */
    public Response<VcInfo> vcGet(String vcId) throws ChainWeaverException {
        Map<String, String> vcMap = new HashMap<>();
        vcMap.put("vcId", vcId);
        Gson gson = new Gson();
        Response<VcInfo> vcInfo;
        try {
            String res = HttpUtils.post(host + DidConfig.vcGet, gson.toJson(vcMap), this.getHeaders(), readTimeOut, connectTimeOut);
            Type type = new TypeToken<Response<VcInfo>>() {
            }.getType();
            vcInfo = gson.fromJson(res, type);
            return vcInfo;
        } catch (IOException e) {
            throw new ChainWeaverException(e);
        }
    }

    /**
     * 更新拥有的VC列表
     *
     * @param issuer 持证人
     * @param limit  限制条数
     * @return 返回本地vc列表
     * @throws ChainWeaverException 访问异常
     */
    public Response<Boolean> vcListLatest(String issuer, Integer limit) throws ChainWeaverException {
        Map<String, Object> online = new HashMap<>();
        online.put("issuer", issuer);
        online.put("limit", limit);
        Gson gson = new Gson();
        Response<Boolean> vcResp;
        try {
            String res = HttpUtils.post(host + DidConfig.vcListLatest, gson.toJson(online), this.getHeaders(), readTimeOut, connectTimeOut);
            Type type = new TypeToken<Response<Boolean>>() {
            }.getType();
            vcResp = gson.fromJson(res, type);
            return vcResp;
        } catch (IOException e) {
            throw new ChainWeaverException(e);
        }
    }


    /**
     * 本地查询拥有VC
     *
     * @param vo {@link VcListVo} 发证人和vcId
     * @return 返回本地vc列表
     * @throws ChainWeaverException 访问异常
     */
    public Response<RespPage<VerifiableCredential>> vcListLocal(VcListVo vo) throws ChainWeaverException {
        Gson gson = new Gson();
        Response<RespPage<VerifiableCredential>> vcResp;
        try {
            String res = HttpUtils.post(host + DidConfig.vcListLocal, gson.toJson(vo), this.getHeaders(), readTimeOut, connectTimeOut);
            Type type = new TypeToken<Response<RespPage<VerifiableCredential>>>() {
            }.getType();
            vcResp = gson.fromJson(res, type);
            return vcResp;
        } catch (IOException e) {
            throw new ChainWeaverException(e);
        }
    }

    /**
     * 申请vct
     *
     * @param vctExtend {@link VCTExtend#VCTExtend(Map properties, List required, String name, String versions, Boolean additionalProperties, String verificationMethod)}
     *                  <li>extend 自定义参数; 可选 </li>
     *                  <ul>
     *                      <li>vcType: 类型;可选 ,default: IdentityCredential </li>
     *                      <li>properties: 自定义字段 Map<String, property>  {@link Property#Property(String type, String title, String reg, String desc)}</li>
     *                                                                          <ul>
     *                                                                              <li>type:  字段类型 </li>
     *                                                                              <li>title: 字段的中文名 </li>
     *                                                                              <li>reg: 正则表达式 </li>
     *                                                                              <li>desc: 描述 </li>
     *                                                                          </ul>
     *                      <li>required：是否必填； List<String> key字段是否必填 </li>
     *                      <li>name: 凭证类型 </li>
     *                      <li>versions： 版本号 </li>
     *                      <li>additionalProperties； Boolean 是否允许VC的credentialSubject添加额外字段 </li>
     *                  <ul/>
     * @return vct id
     * @throws ChainWeaverException 访问异常
     */
    public Response<VctId> vctemplateCreate(VCTExtend vctExtend) throws ChainWeaverException {
        Gson gson = new Gson();
        Response<VctId> vctId;
        try {
            String res = HttpUtils.post(host + DidConfig.vctemplateCreate, gson.toJson(vctExtend), this.getHeaders(), readTimeOut, connectTimeOut);
            Type type = new TypeToken<Response<VctId>>() {
            }.getType();
            vctId = gson.fromJson(res, type);
            return vctId;
        } catch (IOException e) {
            throw new ChainWeaverException(e);
        }
    }

    /**
     * 查询vct
     *
     * @param vcTemplateListVo {@link VcTemplateListVo#VcTemplateListVo(Integer page, Integer size, String id, String name, String version)}
     *                         <ul>
     *                             <li>page-页数</li>
     *                             <li>size-条数.</li>
     *                            <li>id-vcId.</li>
     *                            <li>name-名称.</li>
     *                            <li>version- 版本号.</li>
     *                         </ul>
     * @return 返回vct列表
     * @throws ChainWeaverException 访问异常
     */
    public Response<RespPage<VcTInfo>> vctemplateList(VcTemplateListVo vcTemplateListVo) throws ChainWeaverException {
        Gson gson = new Gson();
        Response<RespPage<VcTInfo>> vcResp;
        try {
            String res = HttpUtils.post(host + DidConfig.vctemplateList, gson.toJson(vcTemplateListVo), this.getHeaders(), readTimeOut, connectTimeOut);
            Type type = new TypeToken<Response<RespPage<VcTInfo>>>() {
            }.getType();
            vcResp = gson.fromJson(res, type);
            return vcResp;
        } catch (IOException e) {
            throw new ChainWeaverException(e);
        }
    }

    /**
     * 生成请求登录VP
     *
     * @param vp {@link VerifiablePresentation#vpCreateApplyLogin(List proof)}
     *           <ul><li>context 上下文, 可选 </li>
     *           <li>id 唯一编号; 可选 </li>
     *           <li>verifiableCredential vcId列表,可验证凭据, 与extend至少有一个不为空; 可选 </li>
     *           <li>extend 自定义参数; 可选 ApplyBusinessLicenseExtend {@link ApplyBusinessLicenseExtend#ApplyBusinessLicenseExtend(List issuer, String callbackUrl)} </li>
     *                  <ul><li>callbackUrl: 回调地址; 必选</li>
     *                  <li>requestId: 请求id; 可选</li>
     *                  <li>method: Http方法(GET/POST)，默认GET; 可选</li></ul>
     *           <li>presentationUsage 用途;可选 </li>
     *           <li>expirationDate 过期时间;可选 </li>
     *           <li>verifier 验证人;可选 </li>
     *           <li>proof 证明结构 {@link Proof#Proof(String type, String created, String proofPurpose, String verificationMethod)}</li>
     *                                      <ul><li>type 签名算法; 可选 ,default: SM2Signature</li>
     *                                      <li>created 签名时间; 可选 </li>
     *                                      <li>proofPurpose 签名目的; 可选 .固定：authentication</li>
     *                                      <li>verificationMethod 签名公钥</li></ul></ul>
     * @return Response<VerifiablePresentation>
     * @throws ChainWeaverException 访问异常
     */
    public Response<VerifiablePresentation<LoginExtend>> vpCreateApplyLogin(VerifiablePresentation<LoginExtend> vp) throws ChainWeaverException {
        return this.vpSign(vp, LoginExtend.class);
    }

    /**
     * 生成请求企业实名VP
     *
     * @param vp {@link VerifiablePresentation#vpCreateApplyBusinessLicense(List proof)}
     *           <ul><li>context 上下文, 可选 </li>
     *           <li>id 唯一编号; 可选 </li>
     *           <li>verifiableCredential vcId列表,可验证凭据, 与extend至少有一个不为空; 可选 </li>
     *           <li>extend 自定义参数; 可选 ApplyBusinessLicenseExtend {@link ApplyBusinessLicenseExtend#ApplyBusinessLicenseExtend(List issuer, String callbackUrl)} </li>
     *                  <ul><li>callbackUrl: 回调地址; 必选</li>
     *                  <li>issuer：发证机构; 可选</li>
     *                  <li>requestId: 请求id; 必选，如果不传，会默认生成一个</li>
     *                  <li>vctId: 模版id; 可选</li>
     *                  <li>method: Http方法(GET/POST)，默认GET; 可选</li></ul>
     *           <li>presentationUsage 用途;必选 </li>
     *           <li>expirationDate 过期时间;可选 </li>
     *           <li>verifier 验证人;可选 </li>
     *           <li>proof 证明结构 {@link Proof#Proof(String type, String created, String proofPurpose, String verificationMethod)}</li>
     *                                      <ul><li>type 签名算法; 可选 ,default: SM2Signature</li>
     *                                      <li>created 签名时间; 可选 </li>
     *                                      <li>proofPurpose 签名目的; 可选 .固定：authentication</li>
     *                                      <li>verificationMethod 签名公钥</li></ul></ul>
     * @return Response<VerifiablePresentation>
     * @throws ChainWeaverException 访问异常
     */
    public Response<VerifiablePresentation<ApplyBusinessLicenseExtend>> vpCreateApplyBusinessLicense(VerifiablePresentation<ApplyBusinessLicenseExtend> vp) throws ChainWeaverException {
        return this.vpSign(vp, ApplyBusinessLicenseExtend.class);
    }


}
