/*
Copyright (C) BABEC. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/
package org.chainweaver.did.sdk.model;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.google.gson.annotations.SerializedName;
import org.chainweaver.did.sdk.model.req.ApplyBusinessLicenseExtend;
import org.chainweaver.did.sdk.model.req.LoginExtend;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * 可验证凭证
 */
public class VerifiablePresentation<T> {

    /**
     * 上下文
     */
    @SerializedName("@context")
    @JsonProperty("@context")
    private List<String> context = Collections.singletonList("https://cnbn.org.cn/ns/vp/v2");

    /**
     * 过期时间
     */
    private String expirationDate;

    /**
     * 自定义参数
     */
    private T extend;

    /**
     * 唯一编号
     */
    private String id;

    /**
     * 用途
     */
    private String presentationUsage;

    /**
     * 证明结构
     */
    private List<Proof> proof;

    /**
     * 时间戳
     */
    private String timestamp;

    /**
     * vp类型
     */
    private String type = "VerifiablePresentation";

    /**
     * Vc 可验证凭证列表
     */
    private List<VerifiableCredential> verifiableCredential;

    /**
     * 验证人
     */
    private String verifier;

    public List<String> getContext() {
        return context;
    }

    public void setContext(List<String> context) {
        this.context = context;
    }

    public String getType() {
        return type;
    }

    public void setType(String type) {
        this.type = type;
    }

    public String getPresentationUsage() {
        return presentationUsage;
    }

    public void setPresentationUsage(String presentationUsage) {
        this.presentationUsage = presentationUsage;
    }

    public List<VerifiableCredential> getVerifiableCredential() {
        return verifiableCredential;
    }

    public void setVerifiableCredential(List<VerifiableCredential> verifiableCredential) {
        this.verifiableCredential = verifiableCredential;
    }

    public String getExpirationDate() {
        return expirationDate;
    }

    public void setExpirationDate(String expirationDate) {
        this.expirationDate = expirationDate;
    }

    public String getVerifier() {
        return verifier;
    }

    public void setVerifier(String verifier) {
        this.verifier = verifier;
    }

    public T getExtend() {
        return extend;
    }

    public void setExtend(T extend) {
        this.extend = extend;
    }

    public List<Proof> getProof() {
        return proof;
    }

    public void setProof(List<Proof> proof) {
        this.proof = proof;
    }

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getTimestamp() {
        return timestamp;
    }

    public void setTimestamp(String timestamp) {
        this.timestamp = timestamp;
    }

    @Override
    public String toString() {
        return "VerifiablePresentation{" +
                "context=" + context +
                ", expirationDate='" + expirationDate + '\'' +
                ", extend=" + extend +
                ", id='" + id + '\'' +
                ", presentationUsage='" + presentationUsage + '\'' +
                ", proof=" + proof +
                ", timestamp='" + timestamp + '\'' +
                ", type='" + type + '\'' +
                ", verifiableCredential=" + verifiableCredential +
                ", verifier='" + verifier + '\'' +
                '}';
    }

    public VerifiablePresentation() {
    }

    /**
     * 签名vp
     *
     * @param verifiableCredential vcId列表,可验证凭据
     * @param proof                证明结构
     */
    public VerifiablePresentation(List<VerifiableCredential> verifiableCredential, List<Proof> proof) {
        this.type = "VerifiablePresentation";
        this.verifiableCredential = verifiableCredential;
        this.expirationDate = this.expirationDate();
        this.proof = proof;
    }

    public VerifiablePresentation(String presentationUsage, List<String> verifiableCredential, List<Proof> proof) {
        this.presentationUsage = presentationUsage;
        this.expirationDate = this.expirationDate();
        this.proof = proof;
    }

    private String expirationDate() {
        // 获取当前时间
        LocalDateTime currentTime = LocalDateTime.now();

        // 加上 5 分钟
        LocalDateTime modifiedTime = currentTime.plusMinutes(5);

        // 格式化成 "2024-03-06T15:06:47Z" 这种格式
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss'Z'");
        return modifiedTime.format(formatter);
    }

    public static VerifiablePresentation<ApplyBusinessLicenseExtend> vpCreateApplyBusinessLicense(List<Proof> proof) {
        return new VerifiablePresentation<>(PresentationUsage.DID_GET_IDENTITY_VC_REQUEST, null, proof);
    }

    public static VerifiablePresentation<LoginExtend> vpCreateApplyLogin(List<Proof> proof) {
        return new VerifiablePresentation<>(PresentationUsage.DID_LOGIN_REQUEST, null, proof);
    }

}
