/*
Copyright (C) BABEC. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/
package org.chainweaver.did.sdk.util;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.net.ssl.*;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.Map;

public class HttpUtils {

    private static final Logger logger = LoggerFactory.getLogger(HttpUtils.class);


    private final static String REQUEST_METHOD = "POST";
    private final static String REQUEST_CONTENT_TYPE = "Content-type";
    private final static String REQUEST_CONTENT_TYPE_BODY = "application/json";

    private final static String REQUEST_CONTENT_TYPE_VALUE = "text/plain";
    private final static String REQUEST_CHARSET = "Accept-Charset";
    private final static String REQUEST_CHARSET_VALUE = "UTF-8";
    private final static String REQUEST_CONTENT_LENGTH = "Content-Length";

    public static String sendGet(String url, int readTimeOut, int connectTimeOut) throws IOException {
        String result = "";
        BufferedReader in = null;
        URL realUrl = new URL(url);
        URLConnection connection = realUrl.openConnection();
        connection.setReadTimeout(readTimeOut);
        connection.setConnectTimeout(connectTimeOut);
        connection.connect();
        in = new BufferedReader(new InputStreamReader(connection.getInputStream(), "utf-8"));
        String line;
        while ((line = in.readLine()) != null) {
            result += line;
        }
        if (in != null) in.close();
        return result;
    }

    public static String post(String apiUrl, String body, Map<String, String> headers, int readTimeOut, int connectTimeOut) throws IOException {
        if (apiUrl.contains("http://")) {
            return httppost(apiUrl, body, headers, readTimeOut, connectTimeOut);
        } else if (apiUrl.contains("https://")) {
            try {
                return httpspost(apiUrl, body, headers);
            } catch (Exception e) {
                e.printStackTrace();
                throw new IOException(e);
            }
        }
        return "";
    }


    public static String httppost(String apiUrl, String body, Map<String, String> headers, int readTimeOut, int connectTimeOut) throws IOException {
        URL url;
        url = new URL(apiUrl);
        HttpURLConnection huc;
        DataOutputStream printout = null;
        huc = (HttpURLConnection) url.openConnection();
        huc.setRequestMethod(REQUEST_METHOD);
        huc.setRequestProperty(REQUEST_CONTENT_TYPE, REQUEST_CONTENT_TYPE_BODY);
//        huc.setRequestProperty(REQUEST_CHARSET, REQUEST_CHARSET_VALUE);
        huc.setRequestProperty(REQUEST_CONTENT_LENGTH, String.valueOf(body.getBytes().length));
        headers.forEach(huc::setRequestProperty);
        huc.setDoOutput(true);
        huc.setReadTimeout(readTimeOut);
        huc.setConnectTimeout(connectTimeOut);
        printout = new DataOutputStream(huc.getOutputStream());
        printout.write(body.getBytes());
        printout.flush();
        if (printout != null) {
            try {
                printout.close();
            } catch (IOException e) {
                logger.error("http close printout error:", e);
            }
        }
        StringBuffer sb = new StringBuffer();
        String line;
        InputStream is = null;
        BufferedReader br = null;
        try {
            is = huc.getInputStream();
            br = new BufferedReader(new InputStreamReader(is));
            while ((line = br.readLine()) != null) {
                sb.append(line).append("\n");
            }
        } catch (IOException e) {
            e.printStackTrace();
            logger.error("http input stream error:", e);
            throw e;
        } finally {
            try {
                if (br != null) {
                    br.close();
                }
                if (is != null) {
                    is.close();
                }
                if (huc != null) {
                    huc.disconnect();
                }
            } catch (IOException e) {
                logger.error("http close input stream error:", e);
                return null;
            }
        }
        return sb.toString();
    }


    public static String httpspost(String apiUrl, String body, Map<String, String> headers)
            throws NoSuchAlgorithmException, KeyManagementException,
            IOException {
        SSLContext sc = SSLContext.getInstance("SSL");
        sc.init(null, new TrustManager[]{new TrustAnyTrustManager()},
                new java.security.SecureRandom());

        URL console = new URL(apiUrl);
        HttpsURLConnection conn = (HttpsURLConnection) console.openConnection();
        conn.setDoOutput(true);
        headers.forEach(conn::setRequestProperty);
        conn.addRequestProperty("connection", "Keep-Alive");
        conn.setRequestMethod("POST");
        conn.setRequestProperty("Content-Type", "application/json");

        conn.setSSLSocketFactory(sc.getSocketFactory());
        conn.setHostnameVerifier(new TrustAnyHostnameVerifier());
        conn.connect();
        DataOutputStream out = new DataOutputStream(conn.getOutputStream());
        out.write(body.getBytes("utf-8"));
        // 刷新、关闭
        out.flush();
        out.close();
        InputStream is = conn.getInputStream();
        if (is != null) {
            ByteArrayOutputStream outStream = new ByteArrayOutputStream();
            byte[] buffer = new byte[1024];
            int len = 0;
            while ((len = is.read(buffer)) != -1) {
                outStream.write(buffer, 0, len);
            }
            is.close();
            if (outStream != null) {
                return new String(outStream.toByteArray(), "utf-8");
            }
        }
        return null;

    }

    private static class TrustAnyTrustManager implements X509TrustManager {

        public void checkClientTrusted(X509Certificate[] chain, String authType)
                throws CertificateException {
        }

        public void checkServerTrusted(X509Certificate[] chain, String authType)
                throws CertificateException {
        }

        public X509Certificate[] getAcceptedIssuers() {
            return new X509Certificate[] {};
        }
    }

    private static class TrustAnyHostnameVerifier implements HostnameVerifier {
        public boolean verify(String hostname, SSLSession session) {
            return true;
        }
    }
}
